/***************************************************************************
 *            client_send_file.c
 *
 *  Fri Jan  9 15:59:42 2004
 *  Copyright  2004  Phil Kett
 *  pkett@genesis-midi.com
 ****************************************************************************/
 #include "../include/remsync.h"
int
client_send_file (char *filename)
{
	int client_sd, result, checksum, size;
	int bytes, inF;
	char message[BIG_BUF], command[BIG_BUF], mode[BIG_BUF], *buffer;
	char fulldest[BIG_BUF];
	struct stat filestat;

	/* First off initialise a connection to the client */
	client_sd = init_client (conf.port, conf.host);
	conv_src_to_dest(conf.source, conf.dest, filename, fulldest);
	/* Stat the file to get file size */
	result = stat (filename, &filestat);
	if (result != 0)
	{
		sprintf (message, "Error: Cannot stat file %s\r\n", filename);
		printf ("%s\n", message);
		return 2;
	}
	size = filestat.st_size;
	checksum = cksum (filename);
	bzero (message, BIG_BUF);
	syslog (LOG_INFO, "Sending file %s\n", filename);
	getperms (filename, mode);
	/* Send command to server to tell it to expect a file */
	/* The command takes the form of recv:filename:filesize:checksum:mode */
	sprintf (message, "recv:%s:%d:%d:%s\r\n", filename, size, checksum, mode);
	syslog(LOG_INFO, "Sent command %s\n", message);
	send (client_sd, message, strlen (message), 0);
	/* Wait for the server to confirm */
	do
	{
		bzero (message, BIG_BUF);
		read (client_sd, message, BIG_BUF);
		strip_string (message);
	}
	while (strcmp (message, "OK") != 0);

	syslog(LOG_INFO, "Got OK, sending file\n");
	
	/* Now we can send the file */
	inF = open (filename, O_RDONLY | O_BINARY);
	buffer = malloc (size);
	bzero (buffer, size);
	bytes = read (inF, buffer, size);
	if (bytes != size)
	{
		logmsg ("file read error");
	}
	bytes = write (client_sd, buffer, size);
	free (buffer);
	close (inF);
	syslog(LOG_INFO, "File Sent\n");
	/* Okay file sent - wait for server to tell us it's got it okay */
	do
	{
		bzero (message, BIG_BUF);
		read (client_sd, message, BIG_BUF);
		strip_string (message);
	}
	while ((strcmp (message, "OK") != 0)
	       && (strcmp (message, "NOTOK") != 0));
	
	client_quit(client_sd);
	
	/* Now check what the server said! */
	if (strcmp (message, "OK") == 0)
	{
		/* The server got the file okay */		
		syslog(LOG_INFO, "File sent OK\n");

		return 0;
	}
	else
	{
		syslog(LOG_INFO, "File send failed\n");
		return -1;
	}
}


int
server_recv_file (char *arguments, int fd)
{
	char filename[BIG_BUF], destfilename[BIG_BUF];
	char asize[BIG_BUF];
	char achecksum[BIG_BUF];
	char mode[BIG_BUF];
	char command[BIG_BUF];
	char local_filename[BIG_BUF];
	char buffer[BIG_BUF];
	char dummy1[BIG_BUF], dummy2[BIG_BUF];
	int size, csum, local_file, sizeread, filemode, total_read, local_csum, rnd,result;
	int block_len = 256;

	split_string (arguments, filename, dummy1);
	split_string (dummy1, asize, dummy2);
	split_string (dummy2, achecksum, mode);


	size = atoi (asize);
	csum = atoi (achecksum);
	filemode = atoi(mode);
	srand(time(NULL));
	rnd = rand ();
	sprintf (local_filename, "%sremsync.%d", conf.dest,rnd);
	local_file =
		open (local_filename, O_CREAT | O_TRUNC | O_WRONLY | O_BINARY,
		      0777);
	if (local_file == (int)NULL)
	{
		return (1);
	}
	sprintf (command, "OK\r\n");
	send (fd, command, strlen (command), 0);

	//printf ("Local File %s\nRemote File %s\nSize %d\n",filename, src_file, size);
	sizeread = 0;
	total_read = 0;
	if (size != 0)
	{
		bzero (buffer, BIG_BUF);
		while ((total_read < size))
		{
			sizeread = read (fd, buffer, block_len);
			total_read += sizeread;
			write (local_file, buffer, sizeread);
		}
	}
	else
	{
		//Handle the case of zero byte files
		// seems to cause problems with the handshaking
		sleep (1);
	}
	close(local_file);
	local_csum = cksum (local_filename);
	if (local_csum != csum)
	{
		printf ("csum Local = %d, Remote = %d\n", local_csum, csum);
		perror ("Error!!! Checksums wrong!");
		sprintf (command, "NOTOK\r\n");
		send (fd, command, strlen (command), 0);
		return (1);
	}
	else
	{
		sprintf (command, "OK\r\n");
		send (fd, command, strlen (command), 0);
		conv_src_to_dest(conf.source, conf.dest, filename, destfilename);
		syslog(LOG_INFO, "Checksums OK, Moving file to %s\n", destfilename);
		result = rename (local_filename, destfilename);
		if (result != 0) {
			syslog(LOG_INFO, "Rename of %s to %s Failed\n", local_filename, destfilename);
		}
		chmod (destfilename, filemode);
		return (0);
	}


}

int server_mdir (char *directory) {
	/* Function to send mdir command to server process */
	int client_socket;
	char message[BIG_BUF], mode[BIG_BUF], fulldest[BIG_BUF];
	struct stat filestat;
	
	/* Initialise connection to server */
	client_socket = init_client(conf.port, conf.host);
	conv_src_to_dest(conf.source, conf.dest, directory, fulldest);
	getperms(directory, mode);
	
	/* Send message to server */
	syslog(LOG_INFO, "Sent mdir message to server\n");
	sprintf(message, "mdir:%s:%s\r\n", fulldest, mode);
	send(client_socket, message, strlen(message), 0);
	/* Wait for server to confirm */
	do
	{
        bzero (message, BIG_BUF);
        read (client_socket, message, BIG_BUF);
        strip_string (message);
    } while (strcmp (message, "OK") != 0);
	/* All done -- close the connection */
	syslog(LOG_INFO, "Got OK from server\n");
	client_quit(client_socket);
	return (0);
}
int server_cmod (char *directory) {
	/* Function to send mdir command to server process */
	int client_socket;
	char message[BIG_BUF], mode[BIG_BUF], fulldest[BIG_BUF];
	struct stat filestat;
	
	/* Initialise connection to server */
	client_socket = init_client(conf.port, conf.host);
	conv_src_to_dest(conf.source, conf.dest, directory, fulldest);
	getperms(directory, mode);
	
	/* Send message to server */
	syslog(LOG_INFO, "Sent cmod message to server\n");
	sprintf(message, "cmod:%s:%s\r\n", fulldest, mode);
	send(client_socket, message, strlen(message), 0);
	/* Wait for server to confirm */
	do
	{
        bzero (message, BIG_BUF);
        read (client_socket, message, BIG_BUF);
        strip_string (message);
    } while (strcmp (message, "OK") != 0);
	/* All done -- close the connection */
	syslog(LOG_INFO, "Got OK from server\n");
	client_quit(client_socket);
	return (0);
}
		
int server_dele (char *filename) {
	/* Function to send dele command to server process */
	int client_socket;
	char message[BIG_BUF], fulldest[BIG_BUF];
	
	/* initialise connection to server */
	client_socket = init_client(conf.port, conf.host);
	
	conv_src_to_dest(conf.source, conf.dest, filename, fulldest);
	/* send message to server */
	
	sprintf(message, "dele:%s\r\n", filename);
	syslog(LOG_INFO, "Sent dele message to server\n");
	send(client_socket, message, strlen(message),0);
	/* Wait for server to confirm */
	do
	{
        bzero (message, BIG_BUF);
        read (client_socket, message, BIG_BUF);
        strip_string (message);
    } while (strcmp (message, "OK") != 0);
	/* All done -- close the connection */
	syslog(LOG_INFO, "Got OK from server\n");
	client_quit(client_socket);
	return (0);
}
